package it.unibo.ai.rec.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;

import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.CombinedDomainXYPlot;
import org.jfree.chart.plot.XYPlot;

import com.fluxicon.slickerbox.factory.SlickerDecorator;
import com.fluxicon.slickerbox.factory.SlickerFactory;

import it.unibo.ai.rec.model.FluentGroup;
import it.unibo.ai.rec.model.FluentsModel;
import it.unibo.ai.rec.model.RecTrace;

public class FluentChartAggregatePanel extends FluentChartContainer {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -373352097104007642L;

	private JScrollPane scrollPane;
	private JCheckBox summaryCheckBox;
	private JPanel container;
	
	public FluentChartAggregatePanel(FluentChartFactory factory) {
		super(factory);
		scrollPane = new JScrollPane();
		scrollPane.setOpaque(false);
		scrollPane.getViewport().setOpaque(false);
		scrollPane.setBorder(BorderFactory.createEmptyBorder());
		scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		SlickerDecorator.instance().decorate(scrollPane.getVerticalScrollBar(),
				new Color(0, 0, 0, 0), new Color(140, 140, 140),
				new Color(80, 80, 80));
		scrollPane.getVerticalScrollBar().setOpaque(false);
		SlickerDecorator.instance().decorate(scrollPane.getHorizontalScrollBar(),
				new Color(0, 0, 0, 0), new Color(140, 140, 140),
				new Color(80, 80, 80));
		scrollPane.getHorizontalScrollBar().setOpaque(false);
		setLayout(new BorderLayout());
		add(scrollPane, BorderLayout.CENTER);
		
		summaryCheckBox = SlickerFactory.instance().createCheckBox("summary", true);
		add(summaryCheckBox, BorderLayout.PAGE_END);
		
		setOpaque(false);
	}

	@Override
	public void update(RecTrace trace, FluentsModel model) {	
		CombinedDomainXYPlot globalPlot = new CombinedDomainXYPlot();
		globalPlot.setDomainAxis(factory.createTraceAxis(trace));
		globalPlot.setGap(0);
		List<String> keys = new ArrayList<String>(model.getFluentGroups().keySet());
		Collections.sort(keys);
		int width = 0;
		int height = 0;
		for(String key: keys) {
			FluentGroup group = model.getFluentGroups().get(key);
			ChartPanel p = factory.getFluentChart(trace, group, summaryCheckBox.isSelected(), false);
			width = p.getPreferredSize().width > width?p.getPreferredSize().width:width;
			height+=p.getPreferredSize().height;
			globalPlot.add((XYPlot)p.getChart().getPlot(), summaryCheckBox.isSelected()?1:group.getFluents().size());
		}
		JFreeChart globalChart = new JFreeChart(null, null, globalPlot, false);
		globalChart.setBackgroundPaint(factory.getGlobalBackgroundPaint());
		ChartPanel globalChartPanel = new ChartPanel(globalChart);
	
		height+= ((HeightEvaluator)globalPlot.getDomainAxis()).getHeight(globalChartPanel);//.getFontMetrics(globalPlot.getDomainAxis().getLabelFont()));
		/*
		for(String groupName: model.getFluentGroups().keySet()) {
			int factor = summaryCheckBox.isSelected()?1:model.getFluentGroups().get(groupName).getFluents().size();
			height += factor*factory.getUnitWidth();
		}

		
		long minDiff = getMinDiff(trace, granularity, metricDistances);
		long maxDiff = last-first;
		
		int width = SwingUtilities.computeStringWidth(globalChartPanel.getFontMetrics(labelFont), longestLabel);
				
		if(metricDistances)	
			width += (int)(maxDiff/minDiff)*unitWidth;
		
		else
			width += (trace.getEvents().size())*unitWidth;*/
		
		/*
		logger.debug("totalHeight: "+height);
		logger.debug("minDiff: "+minDiff);
		logger.debug("maxDiff: "+maxDiff);
		logger.debug("elemsNo: "+maxDiff/minDiff);
		logger.debug("width: "+width);
		*/
		
		
		Dimension d = new Dimension(width,height);
		globalChartPanel.setMaximumDrawWidth(width);
		globalChartPanel.setMinimumDrawWidth(width);
		globalChartPanel.setMaximumDrawHeight(height);
		globalChartPanel.setMinimumDrawHeight(height);
		globalChartPanel.setPreferredSize(d);
		globalChartPanel.setMaximumSize(d);
		globalChartPanel.setMinimumSize(d);
		
		
		
		
		
		
		scrollPane.getViewport().removeAll();
		container = new JPanel();
		container.setLayout(new BoxLayout(container, BoxLayout.LINE_AXIS));
		container.add(globalChartPanel);
		container.setOpaque(false);
		scrollPane.getViewport().add(container);
	}

	@Override
	public JPanel getChartPanel() {
		return container;
	}

}
