package it.unibo.ai.rec.ui;

import it.unibo.ai.rec.model.FluentGroup;
import it.unibo.ai.rec.model.FluentsModel;
import it.unibo.ai.rec.model.RecTrace;
import it.unibo.ai.rec.model.TrendMetric;

import javax.swing.*;

import org.jfree.chart.ChartPanel;
import org.jfree.chart.plot.XYPlot;

import com.fluxicon.slickerbox.factory.SlickerDecorator;

import java.awt.*;
import java.util.*;

public class FluentChartSeparatedPanel extends FluentChartContainer {

	
	private static int METRIC_CHART_HEIGHT = 100;
	private static Color metricColor = new Color(47,172,234);
	private JPanel globalContainer;
	private Map<String, FluentChartSwitchPanel> groupPanels;
	private int maximumLabelWidth = 0;
	private TrendMetric metric;
	private JPanel metricPanel;
	private boolean showCommand;
	
	public FluentChartSeparatedPanel(FluentChartFactory factory) {
		this(factory, null, true);
	}
	
	public FluentChartSeparatedPanel(FluentChartFactory factory, boolean showCommand) {
		this(factory, null, showCommand);
	}
	
	public FluentChartSeparatedPanel(FluentChartFactory factory, TrendMetric metric) {
		this(factory, metric, true);
	}
	
	
	public FluentChartSeparatedPanel(FluentChartFactory factory, TrendMetric metric,boolean showCommand) {
		super(factory);
		this.showCommand = showCommand;
		this.metric = metric;
		setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
		setOpaque(false);
		groupPanels = new TreeMap<String, FluentChartSwitchPanel>(); 
		globalContainer = new JPanel();
		globalContainer.setOpaque(false);
		globalContainer.setLayout(new BoxLayout(globalContainer, BoxLayout.PAGE_AXIS));
		JScrollPane scroller = createSlickerScrollPane();
		JPanel globalWrapper = new JPanel();
		globalWrapper.setLayout(new BoxLayout(globalWrapper, BoxLayout.LINE_AXIS));
		globalWrapper.add(globalContainer);
		globalWrapper.setOpaque(false);
		scroller.getViewport().add(globalContainer);
		add(scroller);
		if(metric != null) {
			metricPanel = new JPanel();
			metricPanel.setOpaque(false);
			metricPanel.setLayout(new BoxLayout(metricPanel, BoxLayout.LINE_AXIS));
			globalContainer.add(metricPanel);
		}
		//add(Box.createVerticalGlue());
		
	}
	
	private JScrollPane createSlickerScrollPane() {
		JScrollPane scrollpane = new JScrollPane();
		scrollpane.setOpaque(false);
		scrollpane.getViewport().setOpaque(false);
		scrollpane.setBorder(BorderFactory.createEmptyBorder());
		//scrollpane.setViewportBorder(BorderFactory.createLineBorder(new Color(
		//		10, 10, 10), 2));
		scrollpane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollpane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		SlickerDecorator.instance().decorate(scrollpane.getVerticalScrollBar(),
				new Color(0, 0, 0, 0), new Color(140, 140, 140),
				new Color(80, 80, 80));
		scrollpane.getVerticalScrollBar().setOpaque(false);
		SlickerDecorator.instance().decorate(scrollpane.getHorizontalScrollBar(),
				new Color(0, 0, 0, 0), new Color(140, 140, 140),
				new Color(80, 80, 80));
		scrollpane.getHorizontalScrollBar().setOpaque(false);		
		return scrollpane;
	}
	
	
	
	private int getMaximumLabelWidth(Collection<String> labels) {
		String longestLabel = "";
		for(String label: labels)
			if(label.length() > longestLabel.length())
				longestLabel = label;
		return SwingUtilities.computeStringWidth(this.getFontMetrics(this.getFont()), longestLabel);
	}
	
	private void setup(Collection<String> groupNames) {
		maximumLabelWidth = getMaximumLabelWidth(groupNames);
		java.util.List<String> myNames = new ArrayList<String>(groupPanels.keySet());
		for(String name: myNames)
			if(!groupNames.contains(name)) {
				FluentChartSwitchPanel p = groupPanels.get(name);
				globalContainer.remove(p);
				groupPanels.remove(name); 
			}
		for(String name: groupNames) {
			registerGroup(name);
			globalContainer.add(groupPanels.get(name));
		}
		//for(FluentChartSwitchPanel p: groupPanels.values())
		//	globalContainer.add(p);
		globalContainer.add(Box.createVerticalGlue());
	}
	
	private void registerGroup(String name) {
		if(groupPanels.get(name) == null) {
			FluentChartSwitchPanel p = new FluentChartSwitchPanel(factory, name, false, showCommand);
			groupPanels.put(name, p);
		}
	}
	
	
	
	@Override
	public void update(RecTrace trace, FluentsModel model) {
		groupPanels.clear();
		globalContainer.removeAll();
		if(metricPanel!=null)
			globalContainer.add(metricPanel);
		//setup(model.getFluentGroups().keySet());
		
		ArrayList<FluentGroup> groups = new ArrayList<FluentGroup>(model.getFluentGroups().values());
		Collections.sort(groups);		
		Vector<String> names = new Vector<String>();
		for(FluentGroup group: groups) {
			names.add(group.getName());
		}
		
		setup(names);
		
		int height = 0;
		int inset = 0;
		/*ArrayList<FluentChartSwitchPanel> panels = new ArrayList<FluentChartSwitchPanel>(groupPanels.values());
		//Collections.sort(panels);
		
		for(int i = 0; i < panels.size(); i++) {
			FluentChartSwitchPanel p = panels.get(i);
			p.update(trace, model);
			if(i == panels.size() - 1) {
				p.setShowAxis(true);
				inset = panels.get(i).getInset();
			}
			else {
				p.setShowAxis(false);
			}
			height += p.getPreferredSize().height;
		}*/
		for(int i = 0; i < names.size(); i++) {
			FluentChartSwitchPanel p = groupPanels.get(names.get(i));
			p.update(trace, model);
			if(i == names.size() - 1) {
				p.setShowAxis(true);
				inset = p.getInset();
			}
			else {
				p.setShowAxis(false);
			}
			height += p.getPreferredSize().height;
		}
		
		globalContainer.add(Box.createVerticalGlue());
		if(metric != null) {
			metricPanel.removeAll();
			ChartPanel metricChartPanel = factory.getMetricChart(trace, model, metric, METRIC_CHART_HEIGHT);
			XYPlot plot = metricChartPanel.getChart().getXYPlot();
			plot.getDomainAxis().setVisible(false);
			plot.getRenderer().setSeriesPaint(0, metricColor);
			plot.getRenderer().setSeriesStroke(0,new BasicStroke(4));
			metricPanel.add(Box.createHorizontalStrut(inset));
			metricPanel.add(metricChartPanel);
			metricPanel.add(Box.createHorizontalGlue());
			height += metricChartPanel.getPreferredSize().height;
		}
		Dimension d = new Dimension(0,height);
		//this.setPreferredSize(d);
		//this.setMinimumSize(d);
		//this.setMaximumSize(d);
		
	}

	
	public JPanel getChartPanel() {
		return globalContainer;
	}
}
